/*******************************************************************
 * リスト3.1 相当の処理で fopen，fread を使うバージョン
 * 本文で説明に用いたリスト３．１では，標準入力からfgetcでファイルを読み込みます．
 * 標準入力がテキストモードで開かれている場合には，
 * バイナリデータ中のデータ(0x1a)をEOFと認識してしまいファイルが正常に読めません．
 *
 * 以下は，fopen，freadを使用したバージョンです．
 * 実行する場合には，引数で，読み込むファイル名を指定します．
 *  ./a.out +src=lenna.bmp
 *******************************************************************/

module bmpinfo;

    integer fp, len, ret;
    reg [7:0] data[2**20-1:0]; // サイズ2^20の8bit幅のデータ配列
    reg [128*8-1:0] src; // 読み込むファイルのファイル名を格納するための変数

    initial begin

        ret = $value$plusargs("src=%s", src); // 実行時に+src=XXXで指定されたXXXを名前として読む
        if(ret == 0) begin // オプションが指定されなかった場合，返戻値は 0 になる．
            $write("usage: ./a.out +src=src_file_name\n");
            $finish;
        end

        fp = $fopen(src,"rb"); // srcに格納された名前のファイルをバイナリモードで開く．
        if(fp == 0) begin // ファイルを開けなかった場合，返戻値は 0 になる．
            $write("Can't open BMP file.\n");
            $finish;
        end

        len = $fread(data, fp); // dataにファイルの内容を格納する．返戻値は読んだ長さ．
        $fclose(fp);

        $write("bfType: %c%c\n", data[0], data[1]);
        $write("bfSize:         %10d\n", {data[5], data[4], data[3], data[2]});
        $write("bfOffBits:      %10d\n", {data[13], data[12], data[11], data[10]});
        $write("biSize:         %10d\n", {data[17], data[16], data[15], data[14]});
        $write("biWidth:        %10d\n", {data[21], data[20], data[19], data[18]});
        $write("biHeight:       %10d\n", {data[25], data[24], data[23], data[22]});
        $write("biPlanes:       %10d\n", {data[27], data[26]});
        $write("biBitCount:     %10d\n", {data[29], data[28]});
        $write("biCompression:  %10d\n", {data[33], data[32], data[31], data[30]});
        $write("biSizeImage:    %10d\n", {data[37], data[36], data[35], data[34]});
        $write("biPixPerMeter:  %10d\n", {data[41], data[40], data[39], data[38]});
        $write("biYPixPerMeter: %10d\n", {data[45], data[44], data[43], data[42]});
        $write("biClrUsed:      %10d\n", {data[49], data[48], data[47], data[46]});
        $write("biCirImportant: %10d\n", {data[53], data[52], data[51], data[50]});
    end

endmodule
