/*******************************************************************
 * リスト3.4 相当の処理で fopen，fread を使うバージョン
 * 本文で説明に用いたリスト３．１では，標準入力からfgetcでファイルを読み込みます．
 * 標準入力がテキストモードで開かれている場合には，
 * バイナリデータ中のデータ(0x1a)をEOFと認識してしまいファイルが正常に読めません．
 *
 * 以下は，fopen，freadを使用したバージョンです．
 * 実行する場合には，引数で，読み込むファイル名を指定します．
 *  ./a.out +src=lenna.bmp
 *******************************************************************/
`define STDIN  (32'h80000000)
`define STDOUT (32'h80000001)
`define EOF    (32'hFFFFFFFF)

module grayscale;

    integer fp;
    reg [31:0] ret, len;
    reg [7:0] data[2**20-1:0];
    reg [128*8-1:0] src; // 読み込むファイルのファイル名を格納するための変数

    reg [31:0] offset, width, height;
    reg [32:0] i = 0;
    reg [16:0] r, g, b;
    reg [16:0] val;
    reg clk  = 1'b0;
    reg done = 1'b0;

    reg [1:0] state = 0; // 何をする状態か，を区別するための変数

    always @(posedge clk) begin
        if(i < width * height) begin
        end else begin
            done <= 1;
        end
        if(state == 0) begin
            b <= {8'h00, data[3*i+offset  ]};
            g <= {8'h00, data[3*i+offset+1]};
            r <= {8'h00, data[3*i+offset+2]};
            state <= 1;
        end else if(state == 1) begin
            val <= (77 * r + 150 * g + 29 * b) >> 8;
            state <= 2;
        end else begin
            data[3*i+offset  ] <= val;
            data[3*i+offset+1] <= val;
            data[3*i+offset+2] <= val;
            i <= i + 1;
            state <= 0;
        end
    end

    initial begin
        clk = 0;

        ret = $value$plusargs("src=%s", src); // 実行時に+src=XXXで指定されたXXXを名前として読む
        if(ret == 0) begin // オプションが指定されなかった場合，返戻値は 0 になる．
            $write("usage: ./a.out +src=src_file_name\n");
            $finish;
        end

        fp = $fopen(src,"rb"); // srcに格納された名前のファイルをバイナリモードで開く．
        if(fp == 0) begin // ファイルを開けなかった場合，返戻値は 0 になる．
            $write("Can't open BMP file.\n");
            $finish;
        end

        len = $fread(data, fp); // dataにファイルの内容を格納する．返戻値は読んだ長さ．
        $fclose(fp);

        offset = {data[13], data[12], data[11], data[10]};
        width  = {data[21], data[20], data[19], data[18]};
        height = {data[25], data[24], data[23], data[22]};
        while(!done) begin
            clk = 0; #10;
            clk = 1; #10;
        end
        for(i = 0; i < len; i = i + 1) begin
            ret = $fputc(data[i], `STDOUT);
        end
        $finish;
    end


endmodule
